"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatters = exports.defaultOptions = void 0;
const events_1 = require("events");
const LogMessage_js_1 = __importDefault(require("./LogMessage.js"));
const logFormatters = __importStar(require("./formatters/index.js"));
const utils_js_1 = require("./utils.js");
const levels = [{
        name: 'fatal',
        method: 'error'
    }, {
        name: 'error',
        method: 'error'
    }, {
        name: 'warn',
        method: 'warn'
    }, {
        name: 'info',
        method: 'info'
    }, {
        name: 'debug',
        method: 'debug'
    }, {
        name: 'trace',
        method: 'debug'
    }];
/**
 * Default options for the LambdaLog class. Stored globally so that it can be modified by the user.
 */
exports.defaultOptions = {
    meta: {},
    tags: [],
    dynamicMeta: null,
    level: 'info',
    dev: false,
    silent: false,
    replacer: null,
    logHandler: console,
    levelKey: '__level',
    messageKey: 'msg',
    tagsKey: '__tags',
    onFormat: logFormatters.json()
};
exports.formatters = logFormatters;
class LambdaLog extends events_1.EventEmitter {
    /**
     * Constructor for the LambdaLog class. Provided to be utilized in more advanced cases to allow overriding and configuration.
     * By default, this module will export an instance of this class, but you may access the class and create your own instance
     * via `log.LambdaLog`.
     * @class
     * @param {LambdaLogOptions} [options={}] Configuration options for LambdaLog.
     */
    constructor(options = {}) {
        super();
        /**
         * Access to the uninstantiated LambdaLog class.
         * @readonly
         */
        this.LambdaLog = LambdaLog;
        /**
         * Access to the uninstantiated LogMessage class.
         */
        this.LogMessage = LogMessage_js_1.default;
        /**
         * Access to the log levels configuration.
         * @readonly
         */
        this.levels = levels;
        this.options = {
            ...exports.defaultOptions,
            ...options
        };
        // Override the max log level from the environment variable
        if (process.env.LAMBDALOG_LEVEL) {
            const lvl = this.getLevel(process.env.LAMBDALOG_LEVEL);
            if (lvl)
                this.options.level = lvl.name;
        }
        // Override the dev setting from the environment variable
        if (process.env.LAMBDALOG_DEV) {
            this.options.dev = (0, utils_js_1.toBool)(process.env.LAMBDALOG_DEV);
        }
        // Override the silent setting from the environment variable
        if (process.env.LAMBDALOG_SILENT) {
            this.options.silent = (0, utils_js_1.toBool)(process.env.LAMBDALOG_SILENT);
        }
    }
    /**
     * Returns the console object to use for logging.
     * @private
     * @returns {ConsoleObject} The configured console object or `console` if none is provided.
     */
    get console() {
        var _a;
        return (_a = this.options.logHandler) !== null && _a !== void 0 ? _a : console;
    }
    /**
     * Generates JSON log message based on the provided parameters and the global configuration. Once the JSON message is created, it is properly logged to the `console`
     * and emitted through an event. If an `Error` or `Error`-like object is provided for `msg`, it will parse out the message and include the stacktrace in the metadata.
     * @throws {Error} If improper log level is provided.
     * @template T The type of the message to log.
     * @param  {string}   level Log level (`info`, `debug`, `warn`, `error`, or `fatal`)
     * @param  {T}        msg   Message to log. Can be any type, but string or `Error` is reccommended.
     * @param  {object|string|number}   [meta={}]  Optional meta data to attach to the log.
     * @param  {string[]} [tags=[]]  Additional tags to append to this log.
     * @returns {LogMessage}  Returns instance of LogMessage.
     */
    _log(level, msg, meta, tags) {
        const lvl = this.getLevel(level);
        if (!lvl) {
            throw new Error(`"${level}" is not a valid log level`);
        }
        // Generate the log message instance
        const message = new this.LogMessage({
            level,
            msg,
            meta,
            tags
        }, this.options);
        // Check if we can log this level
        if (lvl.idx <= this.maxLevelIdx) {
            const consoleObj = this.console;
            // Log the message to the console
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call
            consoleObj[lvl.method](message.toString());
            /**
             * The log event is emitted (using EventEmitter) for every log generated. This allows for custom integrations, such as logging to a thrid-party service.
             * This event is emitted with the [LogMessage](#logmessage) instance for the log. You may control events using all the methods of EventEmitter.
             * @event LambdaLog#log
             * @type {LogMessage}
             */
            this.emit('log', message);
        }
        return message;
    }
    /**
     * Logs a message at the `trace` log level.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    trace(msg, meta, tags) {
        return this._log('trace', msg, meta, tags);
    }
    /**
     * Logs a message at the `debug` log level.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    debug(msg, meta, tags) {
        return this._log('debug', msg, meta, tags);
    }
    /**
     * Logs a message at the `info` log level.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    info(msg, meta, tags) {
        return this._log('info', msg, meta, tags);
    }
    /**
     * Alias for `info`.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    log(msg, meta, tags) {
        return this._log('info', msg, meta, tags);
    }
    /**
     * Logs a message at the `warn` log level.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    warn(msg, meta, tags) {
        return this._log('warn', msg, meta, tags);
    }
    /**
     * Logs a message at the `error` log level.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    error(msg, meta, tags) {
        return this._log('error', msg, meta, tags);
    }
    /**
     * Logs a message at the `error` log level.
     * @template T The type of the message to log.
     * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
     * @param {Metadata} [meta] Optional meta data to attach to the log.
     * @param {Tag[]} [tags] Additional tags to append to this log.
     * @returns {LogMessage} Returns instance of LogMessage.
     */
    fatal(msg, meta, tags) {
        return this._log('fatal', msg, meta, tags);
    }
    /**
     * Generates a log message if `test` is a falsy value. If `test` is truthy, the log message is skipped and returns `false`. Allows creating log messages without the need to
     * wrap them in an if statement. The log level will be `error`.
     * @template T The type of the message to log.
     * @param  {*}        test  Value to test for a falsy value.
     * @param  {T}        msg   Message to log. Can be any type, but string or `Error` is reccommended.
     * @param  {object}   [meta={}]  Optional meta data to attach to the log.
     * @param  {string[]} [tags=[]]  Additional tags to append to this log.
     * @returns {LogMessage|false} The generated log message or `false` if assertion passed.
     */
    assert(test, msg, meta, tags) {
        if (test)
            return false;
        return this._log('error', msg, meta, tags);
    }
    /**
     * Generates a log message with the result or error provided by a promise. Useful for debugging and testing.
     * @param  {Promise<*>} promise  Promise to log the results of.
     * @param  {object}   [meta={}]  Optional meta data to attach to the log.
     * @param  {string[]} [tags=[]]  Additional tags to append to this log.
     * @returns {Promise<LogMessage>} A Promise that resolves with the log message.
     */
    async result(promise, meta, tags) {
        const wrapper = new Promise(resolve => {
            promise
                .then(value => {
                resolve(this._log('info', value, meta, tags));
            })
                .catch(err => {
                resolve(this._log('error', err, meta, tags));
            });
        });
        return wrapper;
    }
    /**
     * Validates and gets the configuration for the provided log level.
     * @private
     * @param {string} level The provided log level string.
     * @returns {object} Returns the configuration for the provided log level.
     */
    getLevel(level) {
        if (!level)
            return false;
        const lvl = levels.findIndex(l => l.name === level.toLowerCase());
        if (lvl === -1)
            return false;
        return {
            idx: lvl,
            ...levels[lvl]
        };
    }
    /**
     * Returns the index of the configured maximum log level.
     * @private
     * @returns {number} The index of the configured maximum log level.
     */
    get maxLevelIdx() {
        if (!this.options.level || this.options.silent)
            return -1;
        return levels.findIndex(l => l.name === this.options.level);
    }
}
exports.default = LambdaLog;
LambdaLog.defaultOptions = exports.defaultOptions;
LambdaLog.formatters = exports.formatters;
