/// <reference types="node" />
declare module "utils" {
    import { StubbedError } from "typings";
    /**
     * Checks if value is an Error or Error-like object
     * @param {any} val The value to check
     * @returns {boolean} Whether the value is an Error or Error-like object
     */
    export function isError(val: unknown): boolean;
    /**
     * Stubs an Error or Error-like object to include a toJSON method.
     * @param {Error} error The error to stub.
     * @returns {StubbedError} The original error object with a toJSON method.
     */
    export function stubError(error: Error): StubbedError;
    /**
     * Converts a string or number value to a boolean.
     * @param {string|number|boolean} val The value to convert.
     * @returns {boolean} The converted value as a boolean.
     */
    export function toBool(val: string | number | boolean): boolean;
}
declare module "formatters/json" {
    import { FormatPlugin } from "typings";
    /**
     * JSON formatter for log messages.
     * @returns {FormatPlugin} The JSON formatter function.
     */
    export default function jsonFormatter(): FormatPlugin;
}
declare module "LogMessage" {
    import { LambdaLogOptions, Message, LogObject, Tag, GenericRecord, StubbedError, Empty, FormatPlugin } from "typings";
    export interface ILogMessage {
        readonly __opts: LambdaLogOptions;
        __level: string;
        __msg: string;
        __meta: GenericRecord;
        __tags: Tag[];
        __error: Error | StubbedError | Empty;
        get level(): string;
        get msg(): string;
        set msg(msg: Message);
        get message(): string;
        set message(msg: Message);
        get meta(): GenericRecord;
        set meta(obj: GenericRecord);
        get tags(): string[];
        set tags(tags: Tag[]);
        get value(): GenericRecord;
        get log(): GenericRecord;
        get throw(): void;
    }
    /**
     * The LogMessage class is a private/internal class that is used for generating log messages. All log methods return an instance of LogMessage allowing for a chainable api.
     * Having a seperate class and instance for each log allows chaining and the ability to further customize this module in the future without major breaking changes. The documentation
     * provided here is what is available to you for each log message.
     */
    export default class LogMessage<MT extends Message = Message> implements ILogMessage {
        readonly __opts: LambdaLogOptions;
        __level: string;
        __msg: string;
        __meta: GenericRecord;
        __tags: Tag[];
        __error: Error | null | undefined;
        /**
         * Constructor for LogMessage
         * @param {LogObject} log The log object to construct the LogMessage from.
         * @param {LambdaLogOptions} opts The options for LambdaLog.
         * @class
         */
        constructor(log: LogObject<MT>, opts: LambdaLogOptions);
        /**
         * String log level of the message.
         * @returns {string} The log level.
         */
        get level(): string;
        /**
         * The message for the log. If an Error was provided, it will be the message of the error.
         * @returns {string} The message for the log.
         */
        get msg(): string;
        /**
         * Update the message for this log to something else.
         * @param {Message} msg The new message for this log.
         */
        set msg(msg: Message);
        /**
         * Alias for `this.msg`.
         * @returns {string} The message for the log.
         */
        get message(): string;
        /**
         * Alias for `this.msg = 'New message';`
         * @param {Message} msg The new message for this log.
         */
        set message(msg: Message);
        /**
         * The fully compiled metadata object for the log. Includes global and dynamic metadata.
         * @returns {GenericRecord} The metadata object.
         */
        get meta(): GenericRecord;
        /**
         * Set additional metadata on the log message.
         * @param {GenericRecord} obj The metadata to add to the log.
         */
        set meta(obj: GenericRecord);
        /**
         * Array of tags attached to this log. Includes global tags.
         * @returns {Tag[]} The tags attached to this log.
         */
        get tags(): string[];
        /**
         * Appends additional tags to this log message.
         * @param {Tag[]} tags The tags to add to this log.
         */
        set tags(tags: Tag[]);
        /**
         * The log represented as an object that is useful for stringifying or pulling data from.
         * @returns {GenericRecord} The compiled log object.
         */
        get value(): GenericRecord<string | number, unknown>;
        /**
         * Alias of `logMessage.value`.
         * @returns {GenericRecord} The compiled log object.
         */
        get log(): GenericRecord<string | number, unknown>;
        /**
         * Throws the log. If an error was not provided, one will be generated for you and thrown. This is useful in cases where you need to log an
         * error, but also throw it.
         * @throws {Error} The provided error, or a newly generated error.
         */
        get throw(): void;
        /**
         * Converts the log to a string using a specific/custom formatter.
         * @returns {string} The formatted log as a string.
         */
        toString(): string;
        /**
         * Converts the log to a string using a specific/custom formatter.
         * @protected
         * @param {FormatPlugin} [formatter] The formatter to use or custom formatter function.
         * @returns {string} The formatted log as a string.
         */
        protected formatMessage(formatter?: FormatPlugin): string;
        /**
         * Takes a log message, parses it, and sets any corresponding properties on the log.
         * @protected
         * @param {Message} message The incoming message.
         */
        protected setMessage(message: Message): void;
        /**
         * Parses a log message and returns any meta, error object, or tags derrived from the message.
         * @protected
         * @param {Message} msg The incoming message.
         * @returns {object}  Object continaing msg, meta, error, and tags.
         */
        protected parseMessage(msg: Message): {
            msg: string;
            meta?: GenericRecord;
            error?: Error;
            tags?: Tag[];
        };
    }
}
declare module "typings" {
    import LogMessage from "LogMessage";
    import stringify from 'fast-safe-stringify';
    export type GenericRecord<K extends string | number = string | number, V = unknown> = Record<K, V>;
    export type Message = string | number | Error;
    export type Metadata = GenericRecord | string | number | null | undefined;
    export type Empty = false | null | undefined;
    type TagFnObject = {
        level: string;
        meta: GenericRecord;
        options: LambdaLogOptions;
    };
    export type Tag = string | number | ((data: TagFnObject) => string | Empty) | Empty;
    type StringifyType = typeof stringify;
    export type LogObject<T extends Message = Message> = {
        level: string;
        msg: T;
        meta?: Metadata;
        tags?: Tag[];
    };
    export type LogLevels = 'fatal' | 'error' | 'warn' | 'info' | 'debug' | 'trace';
    export type ParsePlugin = (msg: Message, options: LambdaLogOptions) => {
        msg: string;
        meta?: GenericRecord;
        error?: Error;
        tags?: Tag[];
    } | Empty;
    export type CompilePlugin = (level?: string, msg?: Message, meta?: GenericRecord, tags?: Tag[], options?: LambdaLogOptions) => GenericRecord;
    export type FormatPlugin = {
        (ctx: LogMessage, options: LambdaLogOptions, stringify: StringifyType): string;
        _cfg?: Record<string, unknown>;
    };
    export type LambdaLogOptions = {
        [key: string]: any;
        meta?: GenericRecord;
        tags?: Tag[];
        dynamicMeta?: ((ctx: LogMessage, options: LambdaLogOptions) => GenericRecord | Empty) | Empty;
        level?: false | LogLevels;
        dev?: boolean;
        silent?: boolean;
        replacer?: ((key: string, val: any) => any) | null | undefined;
        logHandler?: ConsoleObject;
        levelKey?: string | false;
        messageKey?: string;
        tagsKey?: string | false;
        onParse?: ParsePlugin;
        onCompile?: CompilePlugin;
        onFormat?: FormatPlugin;
    };
    export interface ConsoleObject extends Console {
        [key: string]: any;
        log: (...params: any) => void;
        info: (...params: any) => void;
        warn: (...params: any) => void;
        error: (...params: any) => void;
        debug: (...params: any) => void;
    }
    export interface StubbedError extends Error {
        [key: string]: any;
        log?: LogMessage;
        toJSON?: () => GenericRecord;
    }
}
declare module "formatters/full" {
    import { FormatPlugin } from "typings";
    import { InspectOptions } from 'util';
    type FullFormatterCfg = {
        includeTimestamp?: boolean;
        formatTimestamp?: (timestamp: Date) => string;
        includeTags?: boolean;
        includeMeta?: boolean;
        separator?: string;
        inspectOptions?: InspectOptions;
    };
    /**
     * Full formatter for log messages.
     * @param {object} cfg  Configuration object for the formatter.
     * @returns {FormatPlugin} The full formatter function.
     */
    export default function fullFormatter(cfg?: FullFormatterCfg): FormatPlugin;
}
declare module "formatters/minimal" {
    import { FormatPlugin } from "typings";
    type MinimalFormatterCfg = {
        includeTimestamp?: boolean;
        formatTimestamp?: (timestamp: Date) => string;
        separator?: string;
    };
    /**
     * Minimal formatter for log messages.
     * @param {object} cfg  Configuration object for the formatter.
     * @returns {FormatPlugin} The minimal formatter function.
     */
    export default function minimalFormatter(cfg?: MinimalFormatterCfg): FormatPlugin;
}
declare module "formatters/index" {
    export { default as json } from "formatters/json";
    export { default as full } from "formatters/full";
    export { default as minimal } from "formatters/minimal";
}
declare module "LambdaLog" {
    import { EventEmitter } from 'events';
    import { LambdaLogOptions, Message, Metadata, LogLevels, Tag } from "typings";
    import LogMessage from "LogMessage";
    import * as logFormatters from "formatters/index";
    /**
     * Default options for the LambdaLog class. Stored globally so that it can be modified by the user.
     */
    export const defaultOptions: LambdaLogOptions;
    export const formatters: typeof logFormatters;
    export default class LambdaLog extends EventEmitter {
        static defaultOptions: LambdaLogOptions;
        static formatters: typeof logFormatters;
        /**
         * Access to the uninstantiated LambdaLog class.
         * @readonly
         */
        readonly LambdaLog: typeof LambdaLog;
        /**
         * Access to the uninstantiated LogMessage class.
         */
        LogMessage: typeof LogMessage;
        /**
         * The options object for this instance of LambdaLog.
         */
        options: LambdaLogOptions;
        /**
         * Access to the log levels configuration.
         * @readonly
         */
        readonly levels: {
            name: string;
            method: string;
        }[];
        /**
         * Returns the console object to use for logging.
         * @private
         * @returns {ConsoleObject} The configured console object or `console` if none is provided.
         */
        private get console();
        /**
         * Constructor for the LambdaLog class. Provided to be utilized in more advanced cases to allow overriding and configuration.
         * By default, this module will export an instance of this class, but you may access the class and create your own instance
         * via `log.LambdaLog`.
         * @class
         * @param {LambdaLogOptions} [options={}] Configuration options for LambdaLog.
         */
        constructor(options?: LambdaLogOptions);
        /**
         * Generates JSON log message based on the provided parameters and the global configuration. Once the JSON message is created, it is properly logged to the `console`
         * and emitted through an event. If an `Error` or `Error`-like object is provided for `msg`, it will parse out the message and include the stacktrace in the metadata.
         * @throws {Error} If improper log level is provided.
         * @template T The type of the message to log.
         * @param  {string}   level Log level (`info`, `debug`, `warn`, `error`, or `fatal`)
         * @param  {T}        msg   Message to log. Can be any type, but string or `Error` is reccommended.
         * @param  {object|string|number}   [meta={}]  Optional meta data to attach to the log.
         * @param  {string[]} [tags=[]]  Additional tags to append to this log.
         * @returns {LogMessage}  Returns instance of LogMessage.
         */
        _log<T extends Message = Message>(level: LogLevels, msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Logs a message at the `trace` log level.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        trace<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Logs a message at the `debug` log level.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        debug<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Logs a message at the `info` log level.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        info<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Alias for `info`.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        log<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Logs a message at the `warn` log level.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        warn<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Logs a message at the `error` log level.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        error<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Logs a message at the `error` log level.
         * @template T The type of the message to log.
         * @param {T} msg Message to log. Can be any type, but string or `Error` is reccommended.
         * @param {Metadata} [meta] Optional meta data to attach to the log.
         * @param {Tag[]} [tags] Additional tags to append to this log.
         * @returns {LogMessage} Returns instance of LogMessage.
         */
        fatal<T extends Message = Message>(msg: T, meta?: Metadata, tags?: Tag[]): LogMessage;
        /**
         * Generates a log message if `test` is a falsy value. If `test` is truthy, the log message is skipped and returns `false`. Allows creating log messages without the need to
         * wrap them in an if statement. The log level will be `error`.
         * @template T The type of the message to log.
         * @param  {*}        test  Value to test for a falsy value.
         * @param  {T}        msg   Message to log. Can be any type, but string or `Error` is reccommended.
         * @param  {object}   [meta={}]  Optional meta data to attach to the log.
         * @param  {string[]} [tags=[]]  Additional tags to append to this log.
         * @returns {LogMessage|false} The generated log message or `false` if assertion passed.
         */
        assert<T extends Message = Message>(test: unknown, msg: T, meta?: Metadata, tags?: Tag[]): LogMessage | false;
        /**
         * Generates a log message with the result or error provided by a promise. Useful for debugging and testing.
         * @param  {Promise<*>} promise  Promise to log the results of.
         * @param  {object}   [meta={}]  Optional meta data to attach to the log.
         * @param  {string[]} [tags=[]]  Additional tags to append to this log.
         * @returns {Promise<LogMessage>} A Promise that resolves with the log message.
         */
        result(promise: Promise<unknown>, meta?: Metadata, tags?: Tag[]): Promise<LogMessage>;
        /**
         * Validates and gets the configuration for the provided log level.
         * @private
         * @param {string} level The provided log level string.
         * @returns {object} Returns the configuration for the provided log level.
         */
        private getLevel;
        /**
         * Returns the index of the configured maximum log level.
         * @private
         * @returns {number} The index of the configured maximum log level.
         */
        private get maxLevelIdx();
    }
}
declare module "lambda-log" {
    import LambdaLog from "LambdaLog";
    import LogMessage from "LogMessage";
    import * as formatters from "formatters/index";
    import * as Types from "typings";
    const _default: LambdaLog;
    export default _default;
    export { LambdaLog, LogMessage, formatters, Types };
}
declare module "index" {
    import lambdaLog from "lambda-log";
    export default lambdaLog;
}
